;;; org-mairix.el - Support for hooking mairix search into Org for different MUAs
;;
;; Copyright (C) 2007 Georg C. F. Greve
;;
;; Author: Georg C. F. Greve <greve at fsfeurope dot org>
;; Keywords: outlines, hypermedia, calendar, wp, email, mairix
;; Purpose: Integrate mairix email searching into Org mode
;; See http://orgmode.org and http://www.rpcurnow.force9.co.uk/mairix/
;; Version: 0.2
;;
;; This file is Free Software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.

;; It is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
;; License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
;; Boston, MA 02110-1301, USA.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(require 'org)

;;; The custom variables

(defgroup org-mairix nil
  "Mairix support/integration in org."
  :tag "Org Mairix"
  :group 'org)

(defcustom org-mairix-threaded-links t
  "Should new links be created as threaded links?
If t, links will be stored as threaded searches.
If nil, links will be stored as non-threaded searches."
  :group 'org-mairix
  :type 'boolean)

(defcustom org-mairix-augmented-links nil
  "Should new links be created as augmenting searches?
If t, links will be stored as augmenting searches.
If nil, links will be stored as normal searches.

Attention: When activating this option, you will need
to remove old articles from your mairix results group
in some other way, mairix will not do it for you."
  :group 'org-mairix
  :type 'boolean)

(defcustom org-mairix-display-hook 'org-mairix-gnus-display-results
  "Hook to call to display the results of a successful mairix search.
Defaults to Gnus, feel free to add your own MUAs or methods."
  :group 'org-mairix
  :type 'hook)

(defcustom org-mairix-executable "mairix"
  "The mairix executable to call. If your paths are set up
correctly, you should not need to change this."
  :group 'org-mairix
  :type 'string)

(defgroup org-mairix-gnus nil
  "Use gnus for mairix support in org."
  :tag "Org Mairix Gnus"
  :group 'org-mairix)

(defcustom org-mairix-gnus-results-group "nnmaildir:mairix"
  "The group that is configured to hold the mairix search results,
which needs to be setup independently of the org-mairix integration,
along with general mairix configuration."
  :group 'org-mairix-gnus
  :type 'string)


;;; The hooks to integrate mairix into org

(org-add-link-type "mairix" 'org-mairix-open)
(add-hook 'org-store-link-functions 'org-mairix-store-link)


;;; Generic org-mairix functions

(defun org-mairix-store-link ()
  "Store a link to the current message as a Mairix search for its
Message ID."

  ;; gnus integration
  (when (memq major-mode '(gnus-summary-mode gnus-article-mode))
    (and (eq major-mode 'gnus-article-mode) (gnus-article-show-summary))
    (let* ((article (gnus-summary-article-number))
	   (header (gnus-summary-article-header article))
	   (from (mail-header-from header))
	   (message-id (mail-header-id header))
	   ;; FIXME: (bzg) we don't need the date
	   ;; (gg) really? but what if people were to use that in their "org-email-link-description" customisation?
	   (date (mail-header-date header))
	   (subject (gnus-summary-subject-string)))
      (org-store-link-props :type "mairix" :from from :subject subject
			    :message-id message-id)
      (setq cpltxt (org-email-link-description))
      (org-store-link-props :link (concat "mairix:"
					  (if org-mairix-threaded-links "t:")
					  (if org-mairix-augmented-links "a:")
					  message-id)
			    :description cpltxt))))

(defun org-mairix-open (path)
  "Function to open mairix link.

We first need to split it into its individual parts, and then
extract the message-id to be passed on to the display function
before call mairix, evaluate the number of matches returned, and
make sure to only call display of mairix succeeded in matching."
  ;; FIXME (bzg) mhh... better use let* construct for arguments,
  ;; message-id and cmdline.
  ;; (gg) tried to do it as best as possible with new parsing, feel free to improve
  (let* ((cmdline org-mairix-executable))
    (if (string< "t:" path)
	(progn (setq path (substring path 2 nil))
	       (setq cmdline (concat cmdline " --threads"))))
    (if (string< "a:" path)
	(progn (setq path (substring path 2 nil))
	       (setq cmdline (concat cmdline " --augment"))))
    (let* ((message-id (org-remove-angle-brackets path)))
      (setq cmdline (concat cmdline " m:" message-id))

      (print cmdline)

      (setq retval (shell-command-to-string
		    (concat cmdline " m:" message-id)))
      (string-match "\[0-9\]+" retval)
      (setq matches (string-to-number (match-string 0 retval)))
      (if (eq matches 0) (message "Link failed: no matches, sorry.")
	(message "Link returned %d matches." matches)
	(run-hook-with-args 'org-mairix-display-hook message-id)))))


;;; Functions necessary for gnus integration

(defun org-mairix-gnus-display-results (message-id)
  "Display results of mairix search in Gnus.

Note: This does not work as cleanly as I would like it to. The
problem being that Gnus should simply reread the group cleanly,
without remembering anything. At the moment it seems to be unable
to do that -- so you're likely to see zombies floating around.

Gnus developers couldn't really help, see:
http://article.gmane.org/gmane.emacs.gnus.general/65248

If you can improve this, please do!"
  (require 'gnus)
  (require 'gnus-sum)
  ;;FIXME (bzg) we need to call Gnus first, don't we?
  ;;(gg) that checks for new news, AFAIK, slowing things down quite a bit is there an "assert" function?
  (funcall (cdr (assq 'gnus org-link-frame-setup)))
  (gnus-group-quick-select-group 0 org-mairix-gnus-results-group)
  (gnus-summary-reselect-current-group t t)
  (gnus-summary-select-article
   nil t t (car (gnus-find-matching-articles "message-id" message-id))))

(provide 'org-mairix)

;;; org-mairix.el ends here
